@@ -157,40 +157,43 @@ soup_client_input_stream_close_fn (GInputStream  *stream,
 						 cancellable, error);
 }
 
-typedef struct {
-	SoupClientInputStream *cistream;
-	gint priority;
-	GCancellable *cancellable;
-	GSimpleAsyncResult *result;
-} CloseAsyncData;
-
-static void
-close_async_data_free (CloseAsyncData *cad)
+static gboolean
+idle_finish_close (gpointer user_data)
 {
-	g_clear_object (&cad->cancellable);
-	g_object_unref (cad->result);
-	g_slice_free (CloseAsyncData, cad);
+	GTask *task = user_data;
+
+	g_task_return_boolean (task, TRUE);
+	g_object_unref (task);
+	return FALSE;
 }
 
 static gboolean
 close_async_ready (SoupMessage *msg, gpointer user_data)
 {
-	CloseAsyncData *cad = user_data;
+	GTask *task = user_data;
+	SoupClientInputStream *cistream = g_task_get_source_object (task);
 	GError *error = NULL;
 
-	if (!soup_message_io_run_until_finish (cad->cistream->priv->msg,
-					       cad->cancellable, &error) &&
+	if (!soup_message_io_run_until_finish (cistream->priv->msg,
+					       g_task_get_cancellable (task),
+					       &error) &&
 	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
 		g_error_free (error);
 		return TRUE;
 	}
 
-	if (error)
-		g_simple_async_result_take_error (cad->result, error);
-	else
-		g_simple_async_result_set_op_res_gboolean (cad->result, TRUE);
-	g_simple_async_result_complete_in_idle (cad->result);
-	close_async_data_free (cad);
+	if (error) {
+		g_task_return_error (task, error);
+		g_object_unref (task);
+		return FALSE;
+	}
+
+	/* Due to a historical accident, SoupSessionAsync relies on us
+	 * waiting one extra cycle after run_until_finish() returns.
+	 * Ugh. FIXME later when it's easier to do.
+	 */
+	soup_add_idle (g_main_context_get_thread_default (),
+		       idle_finish_close, task);
 	return FALSE;
 }
 
@@ -201,22 +204,17 @@ soup_client_input_stream_close_async (GInputStream        *stream,
 				      GAsyncReadyCallback  callback,
 				      gpointer             user_data)
 {
-	CloseAsyncData *cad;
+	SoupClientInputStream *cistream = SOUP_CLIENT_INPUT_STREAM (stream);
+	GTask *task;
 	GSource *source;
 
-	cad = g_slice_new (CloseAsyncData);
-	cad->cistream = SOUP_CLIENT_INPUT_STREAM (stream);
-	cad->result = g_simple_async_result_new (G_OBJECT (stream),
-						 callback, user_data,
-						 soup_client_input_stream_close_async);
-	cad->priority = priority;
-	cad->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+	task = g_task_new (stream, cancellable, callback, user_data);
+	g_task_set_priority (task, priority);
 
-	source = soup_message_io_get_source (cad->cistream->priv->msg,
-					     cancellable,
-					     close_async_ready, cad);
-	g_source_set_priority (source, priority);
-	g_source_attach (source, g_main_context_get_thread_default ());
+	source = soup_message_io_get_source (cistream->priv->msg,
+					     cancellable, NULL, NULL);
+					     
+	g_task_attach_source (task, source, (GSourceFunc) close_async_ready);
 	g_source_unref (source);
 }
 
@@ -225,12 +223,7 @@ soup_client_input_stream_close_finish (GInputStream  *stream,
 				       GAsyncResult  *result,
 				       GError       **error)
 {
-	GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
-
-	if (g_simple_async_result_propagate_error (simple, error))
-		return FALSE;
-	else
-		return g_simple_async_result_get_op_res_gboolean (simple);
+	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 static void
