@@ -15,7 +15,6 @@ G_BEGIN_DECLS
 
 typedef enum {
 	SOUP_MESSAGE_STARTING,
-	SOUP_MESSAGE_AWAITING_CONNECTION,
 	SOUP_MESSAGE_GOT_CONNECTION,
 	SOUP_MESSAGE_CONNECTING,
 	SOUP_MESSAGE_CONNECTED,
@@ -41,7 +40,8 @@ struct _SoupMessageQueueItem {
 	SoupAddress *proxy_addr;
 	SoupURI *proxy_uri;
 	SoupConnection *conn;
-	GSimpleAsyncResult *result;
+	GTask *task;
+	GSource *io_source;
 
 	guint paused            : 1;
 	guint new_api           : 1;
