@@ -19,16 +19,12 @@ typedef enum {
 	NTLM_AUTHENTICATED_BOB
 } NTLMServerState;
 
-static const char *state_name[] = {
-	"unauth", "recv", "sent", "alice", "bob"
-};
-
 #define NTLM_REQUEST_START "TlRMTVNTUAABAAAA"
 #define NTLM_RESPONSE_START "TlRMTVNTUAADAAAA"
 
 #define NTLM_CHALLENGE "TlRMTVNTUAACAAAADAAMADAAAAABAoEAASNFZ4mrze8AAAAAAAAAAGIAYgA8AAAARABPAE0AQQBJAE4AAgAMAEQATwBNAEEASQBOAAEADABTAEUAUgBWAEUAUgAEABQAZABvAG0AYQBpAG4ALgBjAG8AbQADACIAcwBlAHIAdgBlAHIALgBkAG8AbQBhAGkAbgAuAGMAbwBtAAAAAAA="
 
-#define NTLM_RESPONSE_USER(response) ((response)[86] == 'E' ? NTLM_AUTHENTICATED_ALICE : ((response)[86] == 'I' ? NTLM_AUTHENTICATED_BOB : NTLM_UNAUTHENTICATED))
+#define NTLM_RESPONSE_USER(response) ((response)[86] == 'E' ? NTLM_AUTHENTICATED_ALICE : NTLM_AUTHENTICATED_BOB)
 
 static void
 clear_state (gpointer connections, GObject *ex_connection)
@@ -138,22 +134,16 @@ server_callback (SoupServer *server, SoupMessage *msg,
 		}
 	}
 
-	debug_printf (2, " (S:%s)", state_name[state]);
 	g_hash_table_insert (connections, socket, GINT_TO_POINTER (state));
 	g_object_weak_ref (G_OBJECT (socket), clear_state, connections);
 }
 
-static gboolean authenticated_ntlm = FALSE;
-
 static void
 authenticate (SoupSession *session, SoupMessage *msg,
 	      SoupAuth *auth, gboolean retrying, gpointer user)
 {
-	if (!retrying) {
+	if (!retrying)
 		soup_auth_authenticate (auth, user, "password");
-		if (g_str_equal (soup_auth_get_scheme_name (auth), "NTLM"))
-			authenticated_ntlm = TRUE;
-	}
 }
 
 typedef struct {
@@ -323,8 +313,7 @@ do_message (SoupSession *session, SoupURI *base_uri, const char *path,
 }
 
 static void
-do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm,
-	       const char *user, gboolean use_builtin_ntlm)
+do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm, const char *user)
 {
 	SoupSession *session;
 	gboolean alice = !g_strcmp0 (user, "alice");
@@ -333,23 +322,13 @@ do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm,
 	gboolean bob_via_ntlm = use_ntlm && bob;
 	gboolean alice_via_basic = !use_ntlm && alice;
 
-	session = soup_test_session_new (SOUP_TYPE_SESSION, NULL);
+	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
+	if (use_ntlm)
+		soup_session_add_feature_by_type (session, SOUP_TYPE_AUTH_NTLM);
 
 	if (user) {
 		g_signal_connect (session, "authenticate",
 				  G_CALLBACK (authenticate), (char *)user);
-		if (use_ntlm && !use_builtin_ntlm)
-			g_setenv ("NTLMUSER", user, TRUE);
-	}
-	if (use_ntlm) {
-		SoupAuthManager *auth_manager;
-		SoupAuth *ntlm;
-
-		soup_session_add_feature_by_type (session, SOUP_TYPE_AUTH_NTLM);
-		auth_manager = SOUP_AUTH_MANAGER (soup_session_get_feature (session, SOUP_TYPE_AUTH_MANAGER));
-		ntlm = g_object_new (SOUP_TYPE_AUTH_NTLM, NULL);
-		soup_auth_manager_use_auth (auth_manager, base_uri, ntlm);
-		g_object_unref (ntlm);
 	}
 
 	/* 1. Server doesn't request auth, so both get_ntlm_prompt and
@@ -357,19 +336,11 @@ do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm,
 	 * if we're using NTLM we'll try that even without the server
 	 * asking.
 	 */
-	authenticated_ntlm = FALSE;
 	do_message (session, base_uri, "/noauth",
 		    FALSE, use_ntlm,
 		    FALSE, FALSE,
 		    SOUP_STATUS_OK);
 
-	if (authenticated_ntlm != (use_ntlm && use_builtin_ntlm)) {
-		debug_printf (1, "  ERROR: %s built-in NTLM support, but authenticate signal %s emitted\n",
-			      use_builtin_ntlm ? "Using" : "Not using",
-			      authenticated_ntlm ? "was" : "wasn't");
-		errors++;
-	}
-
 	/* 2. Server requires auth as Alice, so it will request that
 	 * if we didn't already authenticate the connection to her in
 	 * the previous step. If we authenticated as Bob in the
@@ -451,150 +422,42 @@ do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm,
 }
 
 static void
-do_ntlm_tests (SoupURI *base_uri, gboolean use_builtin_ntlm)
+do_ntlm_tests (SoupURI *base_uri)
 {
 	debug_printf (1, "Round 1: Non-NTLM Connection, no auth\n");
-	do_ntlm_round (base_uri, FALSE, NULL, use_builtin_ntlm);
+	do_ntlm_round (base_uri, FALSE, NULL);
 	debug_printf (1, "Round 2: NTLM Connection, user=alice\n");
-	do_ntlm_round (base_uri, TRUE, "alice", use_builtin_ntlm);
+	do_ntlm_round (base_uri, TRUE, "alice");
 	debug_printf (1, "Round 3: NTLM Connection, user=bob\n");
-	do_ntlm_round (base_uri, TRUE, "bob", use_builtin_ntlm);
+	do_ntlm_round (base_uri, TRUE, "bob");
 	debug_printf (1, "Round 4: Non-NTLM Connection, user=alice\n");
-	do_ntlm_round (base_uri, FALSE, "alice", use_builtin_ntlm);
-}
-
-static void
-retry_test_authenticate (SoupSession *session, SoupMessage *msg,
-			 SoupAuth *auth, gboolean retrying,
-			 gpointer user_data)
-{
-	gboolean *retried = user_data;
-
-	if (!retrying) {
-		/* server_callback doesn't actually verify the password,
-		 * only the username. So we pass an incorrect username
-		 * rather than an incorrect password.
-		 */
-		soup_auth_authenticate (auth, "wrong", "password");
-	} else if (!*retried) {
-		soup_auth_authenticate (auth, "alice", "password");
-		*retried = TRUE;
-	}
-}
-
-static void
-do_retrying_test (SoupURI *base_uri)
-{
-	SoupSession *session;
-	SoupMessage *msg;
-	SoupURI *uri;
-	gboolean retried = FALSE;
-
-	debug_printf (1, "  /alice\n");
-
-	session = soup_test_session_new (SOUP_TYPE_SESSION,
-					 SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_AUTH_NTLM,
-					 NULL);
-	g_signal_connect (session, "authenticate",
-			  G_CALLBACK (retry_test_authenticate), &retried);
-
-	uri = soup_uri_new_with_base (base_uri, "/alice");
-	msg = soup_message_new_from_uri ("GET", uri);
-	soup_uri_free (uri);
-
-	soup_session_send_message (session, msg);
-
-	if (!retried) {
-		debug_printf (1, "    Didn't retry!\n");
-		errors++;
-	}
-	if (msg->status_code != SOUP_STATUS_OK) {
-		debug_printf (1, "    Unexpected final status %d %s\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
-	g_object_unref (msg);
-
-	soup_test_session_abort_unref (session);
-
-	debug_printf (1, "  /bob\n");
-
-	session = soup_test_session_new (SOUP_TYPE_SESSION,
-					 SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_AUTH_NTLM,
-					 NULL);
-	g_signal_connect (session, "authenticate",
-			  G_CALLBACK (retry_test_authenticate), &retried);
-	retried = FALSE;
-
-	uri = soup_uri_new_with_base (base_uri, "/bob");
-	msg = soup_message_new_from_uri ("GET", uri);
-	soup_uri_free (uri);
-
-	soup_session_send_message (session, msg);
-
-	if (!retried) {
-		debug_printf (1, "    Didn't retry!\n");
-		errors++;
-	}
-	if (msg->status_code != SOUP_STATUS_UNAUTHORIZED) {
-		debug_printf (1, "    Unexpected final status %d %s\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
-	g_object_unref (msg);
-
-	soup_test_session_abort_unref (session);
+	do_ntlm_round (base_uri, FALSE, "alice");
 }
 
 int
 main (int argc, char **argv)
 {
+	GMainLoop *loop;
 	SoupServer *server;
 	GHashTable *connections;
 	SoupURI *uri;
 
 	test_init (argc, argv, NULL);
 
-	server = soup_test_server_new (TRUE);
+	server = soup_test_server_new (FALSE);
 	connections = g_hash_table_new (NULL, NULL);
 	soup_server_add_handler (server, NULL,
 				 server_callback, connections, NULL);
 
+	loop = g_main_loop_new (NULL, TRUE);
+
 	uri = soup_uri_new ("http://127.0.0.1/");
 	soup_uri_set_port (uri, soup_server_get_port (server));
-
-	/* Built-in NTLM auth support. (We set SOUP_NTLM_AUTH_DEBUG to
-	 * an empty string to ensure that the built-in support is
-	 * being used, even if /usr/bin/ntlm_auth is available.)
-	 */
-	g_setenv ("SOUP_NTLM_AUTH_DEBUG", "", TRUE);
-	debug_printf (1, "Built-in NTLM support\n");
-	do_ntlm_tests (uri, TRUE);
-
-#ifdef USE_NTLM_AUTH
-	/* Samba winbind /usr/bin/ntlm_auth helper support (via a
-	 * helper program that emulates its interface).
-	 */
-	g_setenv ("SOUP_NTLM_AUTH_DEBUG", BUILDDIR "/ntlm-test-helper", TRUE);
-	debug_printf (1, "\nExternal helper support\n");
-	do_ntlm_tests (uri, FALSE);
-
-	/* Support for when ntlm_auth is installed, but the user has
-	 * no cached credentials (and thus we have to fall back to
-	 * libsoup's built-in NTLM support).
-	 */
-	g_setenv ("SOUP_NTLM_AUTH_DEBUG_NOCREDS", "1", TRUE);
-	debug_printf (1, "\nExternal -> fallback support\n");
-	do_ntlm_tests (uri, TRUE);
-#endif
-
-	/* Other tests */
-	g_setenv ("SOUP_NTLM_AUTH_DEBUG", "", TRUE);
-	debug_printf (1, "\nRetrying on failed password\n");
-	do_retrying_test (uri);
-
+	do_ntlm_tests (uri);
 	soup_uri_free (uri);
 
+	g_main_loop_unref (loop);
+
 	soup_test_server_quit_unref (server);
 	test_cleanup ();
 	g_hash_table_destroy (connections);
